;|===========================================================================|
;|                           KEYPAD SCANNING                                 |
;| This routine scans the keypad in order to detect a depressed key. A key   |
;| is only valid when there is any key depression for more than 60 ms and it |
;| makes contact for more than 40 ms without any other keypad entry. The     |
;| input to port A is configured with a pull_up resistor for each of the 4   |
;| MSB's and open drain output for the 4 LSB's.                              |
;|===========================================================================|

;* This section scans and detects a keypad entry.

scan_key res k_dep,key_flag
         ldi dra,00001110b           ; Reset the first row.
scan_k0  ld a,dra                    ; Read the keypad.
         com a
         andi a,11110000b            ; Mask the 4 LSB's.
         jrz scan_k1                 ; Test if a key is depressed.
         jp scan_k4
scan_k1  ld a,dra
         jrr 3,a,scan_k2             ; Test if all rows are reset.
         rlc a                       ; Reset the next row.
         andi a,00001111b            ; Mask the 4 MSB's.
         ld dra,a
         jp scan_k0
scan_k2  jrs k_dep,key_flag,end_scan ; Test if a key has been depressed during
                                     ; the keypad scanning in order to avoid
                                     ; clearing the depressed key counter.
         ld a,krel_cnt
         cpi a,tempo_60              ; 60 ms delay without any key.
         jrc scan_k3
         set k_on,key_flag           ; Allows processing of a key.
scan_k3	 clr kdep_cnt
         res k_sav,key_flag
         clr key_sav 
end_scan ret

;* This section processes a key in order to detect if the key is present for
;* more than 40 ms . It detects if there is more than one keypad entry, it
;* cannot detect if there is more than one key on the same row (key 5 and
;* key 6 for example). Compare key_val register with key code table in order
;* to detect if there is more than one keypad entry on the same row.

scan_k4	 set k_dep,key_flag          ; Set depressed key flag.
         clr krel_cnt                ; Cleared by each depressed key.
         jrr k_on,key_flag,scan_k1   ; Test if there is 60 ms between 2 keys.
         jrs k_sav,key_flag,scan_k5  ; Test if a key has already been saved.
         ld a,dra                    ; Save the depressed key.
         ld key_sav,a
         set k_sav,key_flag          ; Indicates that a key is saved.
         jp scan_k1
scan_k5  ld a,dra                    ; Read the keypad.
         cp a,key_sav                ; Compares the keypad entry with the 
                                     ; key_sav register.
         jrz scan_k6
         res k_on,key_flag           ; Inhibits key processing.
         jp scan_k3
scan_k6	 ld a,kdep_cnt               ; Ensures that the key is depressed for
                                     ; more than 40 ms.
         cpi a,tempo_40
         jrnc scan_k7
         jp scan_k1
scan_k7  ld a,key_sav
         ld key_val,a                ; Save the valid key.
         res k_sav,key_flag          ; Reset k_sav flag in order to process 
                                     ; an another key.
         clr kdep_cnt
         res k_on,key_flag           ; Inhibits key processing.
         jp scan_k1
